from .aggregated_tasks import *
from .bitext_mining import *
from .classification import *
from .clustering import *
from .image_text_pair_classification import *
from .instruction_reranking import *
from .instruction_retrieval import *
from .multichoice import *
from .multilabel_classification import *
from .pair_classification import *
from .regression import *
from .reranking import *
from .retrieval import *
from .sts import *
from .summarization import *
from .zeroshot_classification import *
