from .dalaj_classification import DalajClassification, DalajClassificationV2
from .swe_rec_classification import SweRecClassification, SweRecClassificationV2
from .swedish_sentiment_classification import (
    SwedishSentimentClassification,
    SwedishSentimentClassificationV2,
)

__all__ = [
    "DalajClassification",
    "DalajClassificationV2",
    "SweRecClassification",
    "SweRecClassificationV2",
    "SwedishSentimentClassification",
    "SwedishSentimentClassificationV2",
]
