from .ara import *
from .ben import *
from .bul import *
from .ces import *
from .dan import *
from .deu import *
from .ell import *
from .eng import *
from .est import *
from .fas import *
from .fil import *
from .fin import *
from .fra import *
from .guj import *
from .heb import *
from .hin import *
from .hrv import *
from .ind import *
from .ita import *
from .jav import *
from .jpn import *
from .kan import *
from .kat import *
from .kor import *
from .kur import *
from .mal import *
from .mar import *
from .mkd import *
from .multilingual import *
from .mya import *
from .nep import *
from .nld import *
from .nob import *
from .ory import *
from .pan import *
from .pol import *
from .por import *
from .ron import *
from .rus import *
from .san import *
from .sin import *
from .slk import *
from .slv import *
from .spa import *
from .ssw import *
from .swa import *
from .swe import *
from .tam import *
from .tel import *
from .tha import *
from .tsn import *
from .tur import *
from .ukr import *
from .urd import *
from .vie import *
from .zho import *
from .zul import *
