from mteb.benchmarks.benchmarks.benchmarks import (
    BEIR,
    BEIR_NL,
    BRIGHT,
    BRIGHT_LONG,
    BRIGHT_V1_1,
    BUILT_MTEB,
    C_MTEB,
    CHEMTEB,
    CHEMTEB_V1_1,
    CODE_RAG,
    ENCODECHKA,
    FA_MTEB,
    FA_MTEB_2,
    HUME,
    JINA_VDR,
    JMTEB_LITE_V1,
    JMTEB_V2,
    KOVIDORE_V2,
    LONG_EMBED,
    MIEB_ENG,
    MIEB_IMG,
    MIEB_LITE,
    MIEB_MULTILINGUAL,
    MTEB_DEU,
    MTEB_EN,
    MTEB_ENG_CLASSIC,
    MTEB_EU,
    MTEB_FRA,
    MTEB_INDIC,
    MTEB_JPN,
    MTEB_KOR,
    MTEB_MAIN_RU,
    MTEB_MINERS_BITEXT_MINING,
    MTEB_NL,
    MTEB_POL,
    MTEB_RETRIEVAL_LAW,
    MTEB_RETRIEVAL_MEDICAL,
    MTEB_RETRIEVAL_WITH_INSTRUCTIONS,
    NANOBEIR,
    R2MED,
    RU_SCI_BENCH,
    SEB,
    VIDORE,
    VIDORE_V2,
    VIDORE_V3,
    VISUAL_DOCUMENT_RETRIEVAL,
    VN_MTEB,
    CoIR,
    MTEB_code,
    MTEB_MAIN_RU_v1_1,
    MTEB_multilingual_v1,
    MTEB_multilingual_v2,
    RAR_b,
)
from mteb.benchmarks.benchmarks.rteb_benchmarks import (
    RTEB_CODE,
    RTEB_ENGLISH,
    RTEB_FINANCE,
    RTEB_FRENCH,
    RTEB_GERMAN,
    RTEB_HEALTHCARE,
    RTEB_JAPANESE,
    RTEB_LEGAL,
    RTEB_MAIN,
)

__all__ = [
    "BEIR",
    "BEIR_NL",
    "BRIGHT",
    "BRIGHT_LONG",
    "BRIGHT_V1_1",
    "BUILT_MTEB",
    "CHEMTEB",
    "CHEMTEB_V1_1",
    "CODE_RAG",
    "C_MTEB",
    "ENCODECHKA",
    "FA_MTEB",
    "FA_MTEB_2",
    "HUME",
    "HUME",
    "JINA_VDR",
    "JMTEB_LITE_V1",
    "JMTEB_V2",
    "KOVIDORE_V2",
    "LONG_EMBED",
    "MIEB_ENG",
    "MIEB_IMG",
    "MIEB_LITE",
    "MIEB_MULTILINGUAL",
    "MTEB_DEU",
    "MTEB_EN",
    "MTEB_ENG_CLASSIC",
    "MTEB_EU",
    "MTEB_FRA",
    "MTEB_INDIC",
    "MTEB_JPN",
    "MTEB_KOR",
    "MTEB_MAIN_RU",
    "MTEB_MINERS_BITEXT_MINING",
    "MTEB_NL",
    "MTEB_POL",
    "MTEB_RETRIEVAL_LAW",
    "MTEB_RETRIEVAL_MEDICAL",
    "MTEB_RETRIEVAL_WITH_INSTRUCTIONS",
    "NANOBEIR",
    "R2MED",
    "RTEB_CODE",
    "RTEB_ENGLISH",
    "RTEB_FINANCE",
    "RTEB_FRENCH",
    "RTEB_GERMAN",
    "RTEB_HEALTHCARE",
    "RTEB_JAPANESE",
    "RTEB_LEGAL",
    "RTEB_MAIN",
    "RU_SCI_BENCH",
    "SEB",
    "VIDORE",
    "VIDORE_V2",
    "VIDORE_V3",
    "VISUAL_DOCUMENT_RETRIEVAL",
    "VN_MTEB",
    "CoIR",
    "MTEB_MAIN_RU_v1_1",
    "MTEB_code",
    "MTEB_multilingual_v1",
    "MTEB_multilingual_v2",
    "RAR_b",
]
