import torch

from mteb.models.model_meta import (
    ModelMeta,
    ScoringFunction,
)
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader

from .bge_models import bge_training_data
from .e5_models import E5_TRAINING_DATA
from .sentence_transformers_models import sent_trf_training_dataset

Haon_Chen__speed_embedding_7b_instruct = ModelMeta(
    loader=sentence_transformers_loader,
    name="Haon-Chen/speed-embedding-7b-instruct",
    model_type=["dense"],
    revision="c167e9a8144b397622ce47b85d9edcdeecef3d3f",
    release_date="2024-10-31",
    languages=["eng-Latn"],
    n_parameters=7110660096,
    n_embedding_parameters=None,
    memory_usage_mb=13563,
    max_tokens=32768.0,
    embed_dim=None,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/Haon-Chen/speed-embedding-7b-instruct",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from="mistralai/Mistral-7B-v0.1",
    superseded_by=None,
    citation="""@article{chen2024little,
    title={Little Giants: Synthesizing High-Quality Embedding Data at Scale},
    author={Chen, Haonan and Wang, Liang and Yang, Nan and Zhu, Yutao and Zhao, Ziliang and Wei, Furu and Dou, Zhicheng},
    journal={arXiv preprint arXiv:2410.18634},
    year={2024}
}""",
)
Gameselo__STS_multilingual_mpnet_base_v2 = ModelMeta(
    name="Gameselo/STS-multilingual-mpnet-base-v2",
    model_type=["dense"],
    revision="449f917af30f590fc31f9ffb226c94f21a2f47b8",
    release_date="2024-06-07",
    languages=[],
    loader=sentence_transformers_loader,
    n_parameters=278043648,
    n_embedding_parameters=192_001_536,
    memory_usage_mb=1061,
    max_tokens=514.0,
    embed_dim=768,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/Gameselo/STS-multilingual-mpnet-base-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets={
        # Source: https://huggingface.co/datasets/Gameselo/monolingual-wideNLI
        # https://huggingface.co/Gameselo/STS-multilingual-mpnet-base-v2/discussions/2
        # SNLI,
        # MNLI,
        # QNLI,
        # WNLI,
        # SciTail
        # Vitamin C
        # Trains on all of MTEB
        "AlphaNLI",
        "RTE3",
        "AmazonPolarityClassification",
        "AmazonReviewsClassification",
        "ArguAna",
        "ArxivClusteringP2P",
        "ArxivClusteringS2S",
        "AskUbuntuDupQuestions",
        "BIOSSES",
        "Banking77Classification",
        "BiorxivClusteringP2P",
        "BiorxivClusteringS2S",
        "CQADupstackRetrieval",
        "ClimateFEVER",
        "DBPedia",
        "EmotionClassification",
        "FEVER",
        "FiQA2018",
        "HotpotQA",
        "ImdbClassification",
        "MTOPDomainClassification",
        "MTOPIntentClassification",
        "MassiveIntentClassification",
        "MassiveScenarioClassification",
        "MedrxivClusteringP2P",
        "MedrxivClusteringS2S",
        "MindSmallReranking",
        "NFCorpus",
        "NQ",
        "QuoraRetrieval",
        "RedditClustering",
        "RedditClusteringP2P",
        "SCIDOCS",
        "SICK-R",
        "STS12",
        "STS13",
        "STS14",
        "STS15",
        "STS16",
        "STSBenchmark",
        "SciDocsRR",
        "SciFact",
        "SprintDuplicateQuestions",
        "StackExchangeClustering",
        "StackExchangeClusteringP2P",
        "StackOverflowDupQuestions",
        "SummEval",
        "TRECCOVID",
        "Touche2020",
        "ToxicConversationsClassification",
        "TweetSentimentExtractionClassification",
        "TwentyNewsgroupsClustering",
        "TwitterSemEval2015",
        "TwitterURLCorpus",
        "MSMARCO",
        "AmazonCounterfactualClassification",
        "STS17",
        "STS22",
    },
    adapted_from="sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
    superseded_by=None,
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)

Hum_Works__lodestone_base_4096_v1 = ModelMeta(
    name="Hum-Works/lodestone-base-4096-v1",
    model_type=["dense"],
    revision="9bbc2d0b57dd2198aea029404b0f976712a7d966",
    release_date="2023-08-25",
    languages=["eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=None,
    n_embedding_parameters=None,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Hum-Works/lodestone-base-4096-v1",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets={
        "MSMARCO",
        "MSMARCOHardNegatives",
        "NanoMSMARCORetrieval",
        "MSMARCO-PL",  # translation not trained on
        "mMARCO-NL",  # translation not trained on
        "NQ",
        "NQHardNegatives",
        "NanoNQRetrieval",
        "NQ-PL",  # translation not trained on
        "NQ-NL",  # translation not trained on
        # not in MTEB
        # "s2orc",
        # "flax-sentence-embeddings/stackexchange_title_body_jsonl",
        # "flax-sentence-embeddings/stackexchange_titlebody_best_voted_answer_jsonl": [
        #     "train"
        # ],
        # "flax-sentence-embeddings/stackexchange_title_best_voted_answer_jsonl": [
        #     "train"
        # ],
        # "flax-sentence-embeddings/stackexchange_titlebody_best_and_down_voted_answer_jsonl": [
        #     "train"
        # ],
        # "sentence-transformers/reddit-title-body",
        # "msmarco",
        # "gooaq",
        # "yahoo_answers_topics",
        # "code_search_net",
        # "search_qa",
        # "eli5",
        # "snli",
        # "multi_nli",
        # "wikihow",
        # "natural_questions",
        # "trivia_qa",
        # "embedding-data/sentence-compression",
        # "embedding-data/flickr30k-captions",
        # "embedding-data/altlex",
        # "embedding-data/simple-wiki",
        # "embedding-data/QQP",
        # "embedding-data/SPECTER",
        # "embedding-data/PAQ_pairs",
        # "embedding-data/WikiAnswers",
        # "sentence-transformers/embedding-training-data",
    },
    adapted_from="hum-lodestone-v1",
    superseded_by=None,
)
Jaume__gemma_2b_embeddings = ModelMeta(
    name="Jaume/gemma-2b-embeddings",
    model_type=["dense"],
    revision="86431f65d7c3f66b2af096c61e614a2958f191f1",
    release_date="2024-06-29",
    languages=[],
    loader=sentence_transformers_loader,
    n_parameters=2506172416,
    n_embedding_parameters=None,
    memory_usage_mb=9560,
    max_tokens=8192.0,
    embed_dim=2048,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/Jaume/gemma-2b-embeddings",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=set(),
    adapted_from="google/gemma-2b",
    superseded_by=None,
)

bilingual_embedding_training_data = {
    "STSBenchmark",
    "STSBenchmarkMultilingualSTS",
    "XNLI",
    # not in mteb
    # SNLI
}

Lajavaness__bilingual_embedding_base = ModelMeta(
    name="Lajavaness/bilingual-embedding-base",
    model_type=["dense"],
    revision="0bfc54bb2aa2666dd84715289c7ef58a95eb4d8d",
    release_date="2024-06-26",
    languages=None,
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        trust_remote_code=True,
    ),
    n_parameters=278043648,
    n_embedding_parameters=192_001_536,
    memory_usage_mb=1061,
    max_tokens=514.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/Lajavaness/bilingual-embedding-base",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=bilingual_embedding_training_data,
    adapted_from="dangvantuan/bilingual_impl",
    superseded_by=None,
    citation="""
    @article{conneau2019unsupervised,
  title={Unsupervised cross-lingual representation learning at scale},
  author={Conneau, Alexis and Khandelwal, Kartikay and Goyal, Naman and Chaudhary, Vishrav and Wenzek, Guillaume and Guzm{\'a}n, Francisco and Grave, Edouard and Ott, Myle and Zettlemoyer, Luke and Stoyanov, Veselin},
  journal={arXiv preprint arXiv:1911.02116},
  year={2019}
}

@article{reimers2019sentence,
   title={Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks},
   author={Nils Reimers, Iryna Gurevych},
   journal={https://arxiv.org/abs/1908.10084},
   year={2019}
}

@article{thakur2020augmented,
  title={Augmented SBERT: Data Augmentation Method for Improving Bi-Encoders for Pairwise Sentence Scoring Tasks},
  author={Thakur, Nandan and Reimers, Nils and Daxenberger, Johannes and Gurevych, Iryna},
  journal={arXiv e-prints},
  pages={arXiv--2010},
  year={2020}
}
""",
)
Lajavaness__bilingual_embedding_large = ModelMeta(
    name="Lajavaness/bilingual-embedding-large",
    model_type=["dense"],
    revision="e83179d7a66e8aed1b3015e98bb5ae234ed89598",
    release_date="2024-06-24",
    languages=["fra-Latn", "eng-Latn"],
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        trust_remote_code=True,
    ),
    n_parameters=559890432,
    n_embedding_parameters=256_002_048,
    memory_usage_mb=2136,
    max_tokens=514.0,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/Lajavaness/bilingual-embedding-large",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=bilingual_embedding_training_data,
    adapted_from="dangvantuan/bilingual_impl",
    superseded_by=None,
    citation="""
    @article{conneau2019unsupervised,
  title={Unsupervised cross-lingual representation learning at scale},
  author={Conneau, Alexis and Khandelwal, Kartikay and Goyal, Naman and Chaudhary, Vishrav and Wenzek, Guillaume and Guzm{\'a}n, Francisco and Grave, Edouard and Ott, Myle and Zettlemoyer, Luke and Stoyanov, Veselin},
  journal={arXiv preprint arXiv:1911.02116},
  year={2019}
}

@article{reimers2019sentence,
   title={Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks},
   author={Nils Reimers, Iryna Gurevych},
   journal={https://arxiv.org/abs/1908.10084},
   year={2019}
}

@article{thakur2020augmented,
  title={Augmented SBERT: Data Augmentation Method for Improving Bi-Encoders for Pairwise Sentence Scoring Tasks},
  author={Thakur, Nandan and Reimers, Nils and Daxenberger, Johannes and Gurevych, Iryna},
  journal={arXiv e-prints},
  pages={arXiv--2010},
  year={2020}
}
""",
)
Lajavaness__bilingual_embedding_small = ModelMeta(
    name="Lajavaness/bilingual-embedding-small",
    model_type=["dense"],
    revision="ed4a1dd814de0db81d4a4e287c296a03194463e3",
    release_date="2024-07-17",
    languages=["fra-Latn", "eng-Latn"],
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        trust_remote_code=True,
    ),
    n_parameters=117653760,
    n_embedding_parameters=96_014_208,
    memory_usage_mb=449,
    max_tokens=512.0,
    embed_dim=384,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/Lajavaness/bilingual-embedding-small",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=bilingual_embedding_training_data,
    adapted_from="dangvantuan/bilingual_impl",
    superseded_by=None,
    citation="""
    @article{conneau2019unsupervised,
  title={Unsupervised cross-lingual representation learning at scale},
  author={Conneau, Alexis and Khandelwal, Kartikay and Goyal, Naman and Chaudhary, Vishrav and Wenzek, Guillaume and Guzm{\'a}n, Francisco and Grave, Edouard and Ott, Myle and Zettlemoyer, Luke and Stoyanov, Veselin},
  journal={arXiv preprint arXiv:1911.02116},
  year={2019}
}

@article{reimers2019sentence,
   title={Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks},
   author={Nils Reimers, Iryna Gurevych},
   journal={https://arxiv.org/abs/1908.10084},
   year={2019}
}

@article{thakur2020augmented,
  title={Augmented SBERT: Data Augmentation Method for Improving Bi-Encoders for Pairwise Sentence Scoring Tasks},
  author={Thakur, Nandan and Reimers, Nils and Daxenberger, Johannes and Gurevych, Iryna},
  journal={arXiv e-prints},
  pages={arXiv--2010},
  year={2020}
}
""",
)
Mihaiii__Bulbasaur = ModelMeta(
    name="Mihaiii/Bulbasaur",
    model_type=["dense"],
    revision="6876f839e18ae36224049a41194a431953f08747",
    release_date="2024-04-27",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=17389824,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=66,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/Mihaiii/Bulbasaur",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,  # source model is GTE-tiny where training data is unknown
    # {"Mihaiii/qa-assistant"},
    adapted_from="Mihaiii/dwsdwass",
    superseded_by=None,
)
Mihaiii__Ivysaur = ModelMeta(
    name="Mihaiii/Ivysaur",
    model_type=["dense"],
    revision="65914d976f45beb4bda7485c39d88865b4ce6554",
    release_date="2024-04-27",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=22713216,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=87,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/Mihaiii/Ivysaur",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,  # source model is GTE-tiny where training data is unknown
    # not MTEB: {"Mihaiii/qa-assistant"},
    adapted_from="Mihaiii/jhjghjgh",
    superseded_by=None,
)
Mihaiii__Squirtle = ModelMeta(
    name="Mihaiii/Squirtle",
    model_type=["dense"],
    revision="5b991da48a9286637a256d4a35aab87a1a57b78a",
    release_date="2024-04-30",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=15615360,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=60,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/Mihaiii/Squirtle",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=bge_training_data,  # source model is bge-base-en-v1.5
    # not MTEB: {"Mihaiii/qa-assistant"},
    adapted_from="Mihaiii/test21",
    superseded_by=None,
)
Mihaiii__Venusaur = ModelMeta(
    name="Mihaiii/Venusaur",
    model_type=["dense"],
    revision="0dc817f0addbb7bab8feeeeaded538f9ffeb3419",
    release_date="2024-04-29",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=15615360,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=60,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/Mihaiii/Venusaur",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,  # source model is unknown
    # {"Mihaiii/qa-assistant"},
    adapted_from="Mihaiii/test14",
    superseded_by=None,
)
Mihaiii__Wartortle = ModelMeta(
    name="Mihaiii/Wartortle",
    model_type=["dense"],
    revision="14caca5253414d38a7d28b62d1b7c30ef3293a87",
    release_date="2024-04-30",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=17389824,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=66,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/Mihaiii/Wartortle",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=bge_training_data,  # distill from  bge-base-en-v1.5
    # {"Mihaiii/qa-assistant"},
    adapted_from="Mihaiii/test22",
    superseded_by=None,
)
Mihaiii__gte_micro = ModelMeta(
    name="Mihaiii/gte-micro",
    model_type=["dense"],
    revision="6fd2397cb9dfa7c901aedf9a2a44d3c888ccafdd",
    release_date="2024-04-21",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=17389824,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=66,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/Mihaiii/gte-micro",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
Mihaiii__gte_micro_v4 = ModelMeta(
    name="Mihaiii/gte-micro-v4",
    model_type=["dense"],
    revision="78e1a4b348f8524c3ab2e3e3475788f5adb8c98f",
    release_date="2024-04-22",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=19164288,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=73,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/Mihaiii/gte-micro-v4",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
OrdalieTech__Solon_embeddings_large_0_1 = ModelMeta(
    name="OrdalieTech/Solon-embeddings-large-0.1",
    model_type=["dense"],
    revision="9f6465f6ea2f6d10c6294bc15d84edf87d47cdef",
    release_date="2023-12-09",
    languages=["fra-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=559890432,
    n_embedding_parameters=256_002_048,
    memory_usage_mb=2136,
    max_tokens=514.0,
    embed_dim=1024,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/OrdalieTech/Solon-embeddings-large-0.1",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from="solon-large-06-BIG",
    superseded_by=None,
)
Omartificial_Intelligence_Space__Arabert_all_nli_triplet_Matryoshka = ModelMeta(
    name="Omartificial-Intelligence-Space/Arabert-all-nli-triplet-Matryoshka",
    model_type=["dense"],
    revision="d0361a36f6fe69febfc8550d0918abab174f6f30",
    release_date="2024-06-16",
    languages=["ara-Arab"],
    loader=sentence_transformers_loader,
    n_parameters=135193344,
    n_embedding_parameters=49_152_000,
    memory_usage_mb=516,
    max_tokens=512.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/Omartificial-Intelligence-Space/Arabert-all-nli-triplet-Matryoshka",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=set(),  # not in MTEB: {"Omartificial-Intelligence-Space/Arabic-NLi-Triplet"},
    adapted_from="aubmindlab/bert-base-arabertv02",
    superseded_by=None,
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)
Omartificial_Intelligence_Space__Arabic_MiniLM_L12_v2_all_nli_triplet = ModelMeta(
    name="Omartificial-Intelligence-Space/Arabic-MiniLM-L12-v2-all-nli-triplet",
    model_type=["dense"],
    revision="6916465c43b984e955aa6dc72851474f0128f428",
    release_date="2024-06-25",
    languages=["ara-Arab"],
    loader=sentence_transformers_loader,
    n_parameters=117653760,
    n_embedding_parameters=96_014_208,
    memory_usage_mb=449,
    max_tokens=512.0,
    embed_dim=384,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/Omartificial-Intelligence-Space/Arabic-MiniLM-L12-v2-all-nli-triplet",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=sent_trf_training_dataset,
    # not in MTEB
    # {"Omartificial-Intelligence-Space/Arabic-NLi-Triplet"},
    adapted_from="sentence-transformers/paraphrase-multilingual-MiniLM-L12-v2",
    superseded_by=None,
)
Omartificial_Intelligence_Space__Arabic_all_nli_triplet_Matryoshka = ModelMeta(
    name="Omartificial-Intelligence-Space/Arabic-all-nli-triplet-Matryoshka",
    model_type=["dense"],
    revision="1ca467cc576bd76666a4d21b24ee43afa914dd10",
    release_date="2024-06-14",
    languages=["ara-Arab"],
    loader=sentence_transformers_loader,
    n_parameters=278043648,
    n_embedding_parameters=192_001_536,
    memory_usage_mb=1061,
    max_tokens=514.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/Omartificial-Intelligence-Space/Arabic-all-nli-triplet-Matryoshka",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=sent_trf_training_dataset,  # derived from
    # not in MTEB:
    # {"Omartificial-Intelligence-Space/Arabic-NLi-Triplet"},
    adapted_from="sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
    superseded_by=None,
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)
Omartificial_Intelligence_Space__Arabic_labse_Matryoshka = ModelMeta(
    name="Omartificial-Intelligence-Space/Arabic-labse-Matryoshka",
    model_type=["dense"],
    revision="ee6d5e33c78ed582ade47fd452a74ea52aa5bfe2",
    release_date="2024-06-16",
    languages=["ara-Arab"],
    loader=sentence_transformers_loader,
    n_parameters=470926848,
    n_embedding_parameters=384_885_504,
    memory_usage_mb=1796,
    max_tokens=512.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/Omartificial-Intelligence-Space/Arabic-labse-Matryoshka",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,  # derived from labSE
    # as well as:
    # {"Omartificial-Intelligence-Space/Arabic-NLi-Triplet"},
    adapted_from="sentence-transformers/LaBSE",
    superseded_by=None,
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)
Omartificial_Intelligence_Space__Arabic_mpnet_base_all_nli_triplet = ModelMeta(
    name="Omartificial-Intelligence-Space/Arabic-mpnet-base-all-nli-triplet",
    model_type=["dense"],
    revision="2628cb641e040f44328195fadcdfb58e6d5cffa7",
    release_date="2024-06-15",
    languages=["ara-Arab"],
    loader=sentence_transformers_loader,
    n_parameters=109486464,
    n_embedding_parameters=23_444_736,
    memory_usage_mb=418,
    max_tokens=514.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/Omartificial-Intelligence-Space/Arabic-mpnet-base-all-nli-triplet",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=sent_trf_training_dataset,
    # not in MTEB:
    # {"Omartificial-Intelligence-Space/Arabic-NLi-Triplet"},
    adapted_from="tomaarsen/mpnet-base-all-nli-triplet",
    superseded_by=None,
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)
Omartificial_Intelligence_Space__Marbert_all_nli_triplet_Matryoshka = ModelMeta(
    name="Omartificial-Intelligence-Space/Marbert-all-nli-triplet-Matryoshka",
    model_type=["dense"],
    revision="ecf3274e164f057c4a3dd70691cae0265d87a9d0",
    release_date="2024-06-17",
    languages=["ara-Arab"],
    loader=sentence_transformers_loader,
    n_parameters=162841344,
    n_embedding_parameters=76_800_000,
    memory_usage_mb=621,
    max_tokens=512.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/Omartificial-Intelligence-Space/Marbert-all-nli-triplet-Matryoshka",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=set(),  # not in MTEB: "Omartificial-Intelligence-Space/Arabic-NLi-Triplet"},
    adapted_from="UBC-NLP/MARBERTv2",
    superseded_by=None,
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)
consciousai__cai_lunaris_text_embeddings = ModelMeta(
    name="consciousAI/cai-lunaris-text-embeddings",
    model_type=["dense"],
    revision="8332c464d13505968ff7a6e2213f36fd8730b4c7",
    release_date="2023-06-22",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=None,
    n_embedding_parameters=31_254_528,
    memory_usage_mb=None,
    max_tokens=512.0,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "Transformers"],
    reference="https://huggingface.co/consciousAI/cai-lunaris-text-embeddings",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from="/root/.cache/torch/sentence_transformers/intfloat_e5-large-v2",
    superseded_by=None,
)
consciousai__cai_stellaris_text_embeddings = ModelMeta(
    name="consciousAI/cai-stellaris-text-embeddings",
    model_type=["dense"],
    revision="c000ec4b29588daf0f4a0b2ad4e72ee807d8efc0",
    release_date="2023-06-23",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=None,
    n_embedding_parameters=None,
    memory_usage_mb=None,
    max_tokens=514.0,
    embed_dim=768,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/consciousAI/cai-stellaris-text-embeddings",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from="/root/.cache/torch/sentence_transformers/sentence-transformers_all-mpnet-base-v1/",
    superseded_by=None,
)

SENTENCE_CROISSANT_TRAINING_DATA = {
    "STS12",
    "STSBenchmark",
    "STSBenchmarkMultilingualSTS",
    "QuoraRetrieval",
    "MSMARCO",
    "STSB",
}
manu__sentence_croissant_alpha_v0_2 = ModelMeta(
    name="manu/sentence_croissant_alpha_v0.2",
    model_type=["dense"],
    revision="4610b8cea65d7dd59e0b04af50753933fe5b29b2",
    release_date="2024-03-15",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=1279887360,
    n_embedding_parameters=65_536_000,
    memory_usage_mb=2441,
    max_tokens=2048.0,
    embed_dim=2048,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/manu/sentence_croissant_alpha_v0.2",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=SENTENCE_CROISSANT_TRAINING_DATA,
    adapted_from="croissantllm/CroissantCool",
    superseded_by="manu/sentence_croissant_alpha_v0.3",
)
manu__sentence_croissant_alpha_v0_3 = ModelMeta(
    name="manu/sentence_croissant_alpha_v0.3",
    model_type=["dense"],
    revision="4ac16754f3d81aba76cc32955dc9ee4122df96eb",
    release_date="2024-04-26",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=1279887360,
    n_embedding_parameters=65_536_000,
    memory_usage_mb=2441,
    max_tokens=2048.0,
    embed_dim=2048,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/manu/sentence_croissant_alpha_v0.3",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=SENTENCE_CROISSANT_TRAINING_DATA,
    adapted_from="croissantllm/CroissantCool-v0.2",
    superseded_by="manu/sentence_croissant_alpha_v0.4",
)
manu__sentence_croissant_alpha_v0_4 = ModelMeta(
    name="manu/sentence_croissant_alpha_v0.4",
    model_type=["dense"],
    revision="0ce6372e6a3c21134dcf26dcde13cca869c767fc",
    release_date="2024-04-27",
    languages=["fra-Latn", "eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=1279887360,
    n_embedding_parameters=65_536_000,
    memory_usage_mb=2441,
    max_tokens=2048.0,
    embed_dim=2048,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/manu/sentence_croissant_alpha_v0.4",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=SENTENCE_CROISSANT_TRAINING_DATA,
    # Not in MTEB: {"manu/embedding_data_v2_100k"},
    adapted_from="croissantllm/CroissantCool-v0.2",
    superseded_by=None,
)
thenlper__gte_base = ModelMeta(
    name="thenlper/gte-base",
    model_type=["dense"],
    revision="c078288308d8dee004ab72c6191778064285ec0c",
    release_date="2023-07-27",
    languages=["eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=109482752,
    n_embedding_parameters=23_440_896,
    memory_usage_mb=209,
    max_tokens=512.0,
    embed_dim=768,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/thenlper/gte-base",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
    citation="""@article{li2023towards,
  title={Towards general text embeddings with multi-stage contrastive learning},
  author={Li, Zehan and Zhang, Xin and Zhang, Yanzhao and Long, Dingkun and Xie, Pengjun and Zhang, Meishan},
  journal={arXiv preprint arXiv:2308.03281},
  year={2023}
}""",
)
thenlper__gte_large = ModelMeta(
    name="thenlper/gte-large",
    model_type=["dense"],
    revision="4bef63f39fcc5e2d6b0aae83089f307af4970164",
    release_date="2023-07-27",
    languages=["eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=335142400,
    n_embedding_parameters=31_254_528,
    memory_usage_mb=639,
    max_tokens=512.0,
    embed_dim=1024,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/thenlper/gte-large",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
    citation="""@article{li2023towards,
  title={Towards general text embeddings with multi-stage contrastive learning},
  author={Li, Zehan and Zhang, Xin and Zhang, Yanzhao and Long, Dingkun and Xie, Pengjun and Zhang, Meishan},
  journal={arXiv preprint arXiv:2308.03281},
  year={2023}
}""",
)
thenlper__gte_small = ModelMeta(
    name="thenlper/gte-small",
    model_type=["dense"],
    revision="17e1f347d17fe144873b1201da91788898c639cd",
    release_date="2023-07-27",
    languages=["eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=33360512,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=64,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/thenlper/gte-small",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
    citation="""@article{li2023towards,
  title={Towards general text embeddings with multi-stage contrastive learning},
  author={Li, Zehan and Zhang, Xin and Zhang, Yanzhao and Long, Dingkun and Xie, Pengjun and Zhang, Meishan},
  journal={arXiv preprint arXiv:2308.03281},
  year={2023}
}""",
)
sdadas__mmlw_e5_base = ModelMeta(
    name="sdadas/mmlw-e5-base",
    model_type=["dense"],
    revision="f10628ed55b5ec400502aff439bd714a6da0af30",
    release_date="2023-11-17",
    languages=["pol-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=278043648,
    n_embedding_parameters=192_001_536,
    memory_usage_mb=1061,
    max_tokens=514.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/sdadas/mmlw-e5-base",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=E5_TRAINING_DATA,
    adapted_from="intfloat/multilingual-e5-base",
    superseded_by=None,
    citation="""@article{dadas2024pirb,
  title={{PIRB}: A Comprehensive Benchmark of Polish Dense and Hybrid Text Retrieval Methods},
  author={Sławomir Dadas and Michał Perełkiewicz and Rafał Poświata},
  year={2024},
  eprint={2402.13350},
  archivePrefix={arXiv},
  primaryClass={cs.CL}
}""",
)
dwzhu__e5_base_4k = ModelMeta(
    name="dwzhu/e5-base-4k",
    model_type=["dense"],
    revision="1b5664b8cb2bccd8c309429b7bfe5864402e8fbc",
    release_date="2024-03-28",
    languages=["eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=None,
    n_embedding_parameters=23_440_896,
    memory_usage_mb=None,
    max_tokens=4096.0,
    embed_dim=None,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/dwzhu/e5-base-4k",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=E5_TRAINING_DATA,
    adapted_from="intfloat/e5-base-v2",
    superseded_by=None,
    citation="""@article{zhu2024longembed,
  title={LongEmbed: Extending Embedding Models for Long Context Retrieval},
  author={Zhu, Dawei and Wang, Liang and Yang, Nan and Song, Yifan and Wu, Wenhao and Wei, Furu and Li, Sujian},
  journal={arXiv preprint arXiv:2404.12096},
  year={2024}
}""",
)
sdadas__mmlw_e5_large = ModelMeta(
    name="sdadas/mmlw-e5-large",
    model_type=["dense"],
    revision="5c143fb045ebed664fd85b43fc45155999eb110f",
    release_date="2023-11-17",
    languages=["pol-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=559890432,
    n_embedding_parameters=256_002_048,
    memory_usage_mb=2136,
    max_tokens=514.0,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/sdadas/mmlw-e5-large",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=E5_TRAINING_DATA,
    adapted_from="intfloat/multilingual-e5-large",
    superseded_by=None,
    citation="""@article{dadas2024pirb,
  title={{PIRB}: A Comprehensive Benchmark of Polish Dense and Hybrid Text Retrieval Methods},
  author={Sławomir Dadas and Michał Perełkiewicz and Rafał Poświata},
  year={2024},
  eprint={2402.13350},
  archivePrefix={arXiv},
  primaryClass={cs.CL}
}""",
)
sdadas__mmlw_e5_small = ModelMeta(
    name="sdadas/mmlw-e5-small",
    model_type=["dense"],
    revision="ff1298cb6d997f18b794d2f3d73cad2ba2ad739a",
    release_date="2023-11-17",
    languages=["pol-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=117653760,
    n_embedding_parameters=96_014_208,
    memory_usage_mb=449,
    max_tokens=512.0,
    embed_dim=384,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/sdadas/mmlw-e5-small",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=E5_TRAINING_DATA,
    adapted_from="intfloat/multilingual-e5-small",
    superseded_by=None,
    citation="""@article{dadas2024pirb,
  title={{PIRB}: A Comprehensive Benchmark of Polish Dense and Hybrid Text Retrieval Methods},
  author={Sławomir Dadas and Michał Perełkiewicz and Rafał Poświata},
  year={2024},
  eprint={2402.13350},
  archivePrefix={arXiv},
  primaryClass={cs.CL}
}""",
)
sdadas__mmlw_roberta_base = ModelMeta(
    name="sdadas/mmlw-roberta-base",
    model_type=["dense"],
    revision="0ac7f23f6c96af601fa6a17852bd08d5136d6365",
    release_date="2023-11-17",
    languages=["pol-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=124442880,
    n_embedding_parameters=38_400_768,
    memory_usage_mb=475,
    max_tokens=514.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/sdadas/mmlw-roberta-base",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets={"MSMARCO"},
    adapted_from="sdadas/polish-roberta-base-v2",
    superseded_by=None,
    citation="""@article{dadas2024pirb,
  title={{PIRB}: A Comprehensive Benchmark of Polish Dense and Hybrid Text Retrieval Methods},
  author={Sławomir Dadas and Michał Perełkiewicz and Rafał Poświata},
  year={2024},
  eprint={2402.13350},
  archivePrefix={arXiv},
  primaryClass={cs.CL}
}""",
)
sdadas__mmlw_roberta_large = ModelMeta(
    name="sdadas/mmlw-roberta-large",
    model_type=["dense"],
    revision="b8058066a8de32d0737b3cd82d8b4f4108745af9",
    release_date="2023-11-17",
    languages=["pol-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=434961408,
    n_embedding_parameters=131_073_024,
    memory_usage_mb=1659,
    max_tokens=514.0,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/sdadas/mmlw-roberta-large",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets={"MSMARCO"},
    adapted_from="sdadas/polish-roberta-large-v2",
    superseded_by=None,
    citation="""@article{dadas2024pirb,
  title={{PIRB}: A Comprehensive Benchmark of Polish Dense and Hybrid Text Retrieval Methods},
  author={Sławomir Dadas and Michał Perełkiewicz and Rafał Poświata},
  year={2024},
  eprint={2402.13350},
  archivePrefix={arXiv},
  primaryClass={cs.CL}
}""",
)

udever_dataset = {  # discussed here: https://github.com/embeddings-benchmark/mteb/issues/2193
    "MSMARCO",
    # SNLI
    # MultiNLI
}

udever_languages = [
    "aka-Latn",
    "ara-Arab",
    "asm-Beng",
    "bam-Latn",
    "ben-Beng",
    "cat-Latn",
    "eng-Latn",
    "spa-Latn",
    "eus-Latn",
    "fon-Latn",
    "fra-Latn",
    "guj-Gujr",
    "hin-Deva",
    "ind-Latn",
    "ibo-Latn",
    "kik-Latn",
    "kan-Knda",
    "lug-Latn",
    "lin-Latn",
    "mal-Mlym",
    "mar-Deva",
    "nep-Deva",
    "nso-Latn",
    "nya-Latn",
    "ori-Orya",
    "pan-Guru",
    "por-Latn",
    "run-Latn",
    "kin-Latn",
    "sna-Latn",
    "sot-Latn",
    "swa-Latn",
    "tam-Taml",
    "tel-Telu",
    "tsn-Latn",
    "tso-Latn",
    "tum-Latn",
    "twi-Latn",
    "urd-Arab",
    "vie-Latn",
    "wol-Latn",
    "xho-Latn",
    "yor-Latn",
    "zho-Hans",
    "zul-Latn",
]

izhx__udever_bloom_1b1 = ModelMeta(
    name="izhx/udever-bloom-1b1",
    model_type=["dense"],
    revision="7bf1ee29878cb040b2708a691aa4b61f27eaa252",
    release_date="2023-10-24",
    languages=udever_languages,
    loader=sentence_transformers_loader,
    n_parameters=None,
    n_embedding_parameters=385_351_680,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=None,
    license="https://huggingface.co/spaces/bigscience/license",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/izhx/udever-bloom-1b1",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=udever_dataset,
    adapted_from="bigscience/bloom-1b1",
    superseded_by=None,
    citation="""@article{zhang2023language,
  title={Language Models are Universal Embedders},
  author={Zhang, Xin and Li, Zehan and Zhang, Yanzhao and Long, Dingkun and Xie, Pengjun and Zhang, Meishan and Zhang, Min},
  journal={arXiv preprint arXiv:2310.08232},
  year={2023}
}""",
)
izhx__udever_bloom_3b = ModelMeta(
    name="izhx/udever-bloom-3b",
    model_type=["dense"],
    revision="4edd8affe80ca89ba0f6b6ba4103fc7f25fc57b2",
    release_date="2023-10-24",
    languages=udever_languages,
    loader=sentence_transformers_loader,
    n_parameters=None,
    n_embedding_parameters=642_252_800,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=None,
    license="https://huggingface.co/spaces/bigscience/license",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/izhx/udever-bloom-3b",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=udever_dataset,
    adapted_from="bigscience/bloom-3b",
    superseded_by=None,
    citation="""@article{zhang2023language,
  title={Language Models are Universal Embedders},
  author={Zhang, Xin and Li, Zehan and Zhang, Yanzhao and Long, Dingkun and Xie, Pengjun and Zhang, Meishan and Zhang, Min},
  journal={arXiv preprint arXiv:2310.08232},
  year={2023}
}""",
)
izhx__udever_bloom_560m = ModelMeta(
    name="izhx/udever-bloom-560m",
    model_type=["dense"],
    revision="b2a723e355946ec5a5c5fbed3459766627ded2bb",
    release_date="2023-10-24",
    languages=udever_languages,
    loader=sentence_transformers_loader,
    n_parameters=None,
    n_embedding_parameters=256_901_120,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=None,
    license="https://huggingface.co/spaces/bigscience/license",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/izhx/udever-bloom-560m",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=udever_dataset,
    adapted_from="bigscience/bloom-560m",
    superseded_by=None,
    citation="""@article{zhang2023language,
  title={Language Models are Universal Embedders},
  author={Zhang, Xin and Li, Zehan and Zhang, Yanzhao and Long, Dingkun and Xie, Pengjun and Zhang, Meishan and Zhang, Min},
  journal={arXiv preprint arXiv:2310.08232},
  year={2023}
}""",
)
izhx__udever_bloom_7b1 = ModelMeta(
    name="izhx/udever-bloom-7b1",
    model_type=["dense"],
    revision="18e8d3e6dbd94868584877f2e72a105a17df22ef",
    release_date="2023-10-24",
    languages=udever_languages,
    loader=sentence_transformers_loader,
    n_parameters=None,
    n_embedding_parameters=1_027_604_480,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=None,
    license="https://huggingface.co/spaces/bigscience/license",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/izhx/udever-bloom-7b1",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=udever_dataset,
    adapted_from="bigscience/bloom-7b1",
    superseded_by=None,
    citation="""@article{zhang2023language,
  title={Language Models are Universal Embedders},
  author={Zhang, Xin and Li, Zehan and Zhang, Yanzhao and Long, Dingkun and Xie, Pengjun and Zhang, Meishan and Zhang, Min},
  journal={arXiv preprint arXiv:2310.08232},
  year={2023}
}""",
)
avsolatorio__gist_embedding_v0 = ModelMeta(
    name="avsolatorio/GIST-Embedding-v0",
    model_type=["dense"],
    revision="bf6b2e55e92f510a570ad4d7d2da2ec8cd22590c",
    release_date="2024-01-31",
    languages=["eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=109482240,
    n_embedding_parameters=23_440_896,
    memory_usage_mb=418,
    max_tokens=512.0,
    embed_dim=768,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/avsolatorio/GIST-Embedding-v0",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets={
        # not in mteb:
        # MEDI
        # all MTEB CLF datasets that has a train split:
        "AmazonPolarityClassification",
        "AmazonReviewsClassification",
        "EmotionClassification",
        "ImdbClassification",
        "MTOPDomainClassification",
        "MTOPIntentClassification",
        "MassiveIntentClassification",
        "MassiveScenarioClassification",
        "ToxicConversationsClassification",
        "TweetSentimentExtractionClassification",
        "Banking77Classification",
        "AmazonCounterfactualClassification",
    }
    | bge_training_data,
    adapted_from="BAAI/bge-large-en-v1.5",
    superseded_by=None,
    citation="""@article{solatorio2024gistembed,
    title={GISTEmbed: Guided In-sample Selection of Training Negatives for Text Embedding Fine-tuning},
    author={Aivin V. Solatorio},
    journal={arXiv preprint arXiv:2402.16829},
    year={2024},
    URL={https://arxiv.org/abs/2402.16829}
    eprint={2402.16829},
    archivePrefix={arXiv},
    primaryClass={cs.LG}
}""",
)
avsolatorio__gist_all_minilm_l6_v2 = ModelMeta(
    name="avsolatorio/GIST-all-MiniLM-L6-v2",
    model_type=["dense"],
    revision="ea89dfad053bba14677bb784a4269898abbdce44",
    release_date="2024-02-03",
    languages=["eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=22713216,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=87,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/avsolatorio/GIST-all-MiniLM-L6-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets={
        # not in mteb:
        # MEDI
        # all MTEB CLF datasets that has a train split:
        "AmazonPolarityClassification",
        "AmazonReviewsClassification",
        "EmotionClassification",
        "ImdbClassification",
        "MTOPDomainClassification",
        "MTOPIntentClassification",
        "MassiveIntentClassification",
        "MassiveScenarioClassification",
        "ToxicConversationsClassification",
        "TweetSentimentExtractionClassification",
        "Banking77Classification",
        "AmazonCounterfactualClassification",
    }
    | bge_training_data,
    adapted_from=None,
    superseded_by=None,
    citation="""@article{solatorio2024gistembed,
    title={GISTEmbed: Guided In-sample Selection of Training Negatives for Text Embedding Fine-tuning},
    author={Aivin V. Solatorio},
    journal={arXiv preprint arXiv:2402.16829},
    year={2024},
    URL={https://arxiv.org/abs/2402.16829}
    eprint={2402.16829},
    archivePrefix={arXiv},
    primaryClass={cs.LG}
}""",
)
avsolatorio__gist_large_embedding_v0 = ModelMeta(
    name="avsolatorio/GIST-large-Embedding-v0",
    model_type=["dense"],
    revision="7831200e2f7819b994490c091cf3258a2b821f0c",
    release_date="2024-02-14",
    languages=["eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=335141888,
    n_embedding_parameters=31_254_528,
    memory_usage_mb=1278,
    max_tokens=512.0,
    embed_dim=1024,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/avsolatorio/GIST-large-Embedding-v0",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets={
        # not in mteb:
        # MEDI
        # all MTEB CLF datasets that has a train split:
        "AmazonPolarityClassification",
        "AmazonReviewsClassification",
        "EmotionClassification",
        "ImdbClassification",
        "MTOPDomainClassification",
        "MTOPIntentClassification",
        "MassiveIntentClassification",
        "MassiveScenarioClassification",
        "ToxicConversationsClassification",
        "TweetSentimentExtractionClassification",
        "Banking77Classification",
        "AmazonCounterfactualClassification",
    }
    | bge_training_data,
    adapted_from=None,
    superseded_by=None,
    citation="""@article{solatorio2024gistembed,
    title={GISTEmbed: Guided In-sample Selection of Training Negatives for Text Embedding Fine-tuning},
    author={Aivin V. Solatorio},
    journal={arXiv preprint arXiv:2402.16829},
    year={2024},
    URL={https://arxiv.org/abs/2402.16829}
    eprint={2402.16829},
    archivePrefix={arXiv},
    primaryClass={cs.LG}
}""",
)
avsolatorio__gist_small_embedding_v0 = ModelMeta(
    name="avsolatorio/GIST-small-Embedding-v0",
    model_type=["dense"],
    revision="d6c4190f9e01b9994dc7cac99cf2f2b85cfb57bc",
    release_date="2024-02-03",
    languages=["eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=33360000,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=127,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/avsolatorio/GIST-small-Embedding-v0",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets={
        # not in mteb:
        # MEDI
        # all MTEB CLF datasets that has a train split:
        "AmazonPolarityClassification",
        "AmazonReviewsClassification",
        "EmotionClassification",
        "ImdbClassification",
        "MTOPDomainClassification",
        "MTOPIntentClassification",
        "MassiveIntentClassification",
        "MassiveScenarioClassification",
        "ToxicConversationsClassification",
        "TweetSentimentExtractionClassification",
        "Banking77Classification",
        "AmazonCounterfactualClassification",
    }
    | bge_training_data,
    adapted_from=None,
    superseded_by=None,
    citation="""@article{solatorio2024gistembed,
    title={GISTEmbed: Guided In-sample Selection of Training Negatives for Text Embedding Fine-tuning},
    author={Aivin V. Solatorio},
    journal={arXiv preprint arXiv:2402.16829},
    year={2024},
    URL={https://arxiv.org/abs/2402.16829}
    eprint={2402.16829},
    archivePrefix={arXiv},
    primaryClass={cs.LG}
}""",
)
bigscience__sgpt_bloom_7b1_msmarco = ModelMeta(
    name="bigscience/sgpt-bloom-7b1-msmarco",
    model_type=["dense"],
    revision="dc579f3d2d5a0795eba2049e16c3e36c74007ad3",
    release_date="2022-08-26",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=None,
    n_embedding_parameters=1_026_793_472,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=4096,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/bigscience/sgpt-bloom-7b1-msmarco",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from="/gpfsscratch/rech/six/commun/commun/experiments/muennighoff/bloomckpt/6b3/bloom-7b1",
    superseded_by=None,
    citation="""@article{muennighoff2022sgpt,
  title={SGPT: GPT Sentence Embeddings for Semantic Search},
  author={Muennighoff, Niklas},
  journal={arXiv preprint arXiv:2202.08904},
  year={2022}
}""",
)
aari1995__german_semantic_sts_v2 = ModelMeta(
    name="aari1995/German_Semantic_STS_V2",
    model_type=["dense"],
    revision="22912542b0ec7a7ef369837e28ffe6352a27afc9",
    release_date="2022-11-17",
    languages=["deu-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=335736320,
    n_embedding_parameters=31_848_448,
    memory_usage_mb=1281,
    max_tokens=512.0,
    embed_dim=1024,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/aari1995/German_Semantic_STS_V2",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,  # couldn't figure out the source model
    # {"stsb_multi_mt"},
    adapted_from="/content/drive/MyDrive/Stanford_NLU/Project/false_friends/gbert_large_sts_only",
    superseded_by=None,
)
abhinand__medembed_small_v0_1 = ModelMeta(
    name="abhinand/MedEmbed-small-v0.1",
    model_type=["dense"],
    revision="40a5850d046cfdb56154e332b4d7099b63e8d50e",
    release_date="2024-10-20",
    languages=["eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=33360000,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=127,
    max_tokens=512.0,
    embed_dim=384,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/abhinand/MedEmbed-small-v0.1",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets={
        "MedicalQARetrieval",
        "NFCorpus",
        "NFCorpus-NL",  # translation not trained on
        "PublicHealthQA",
        "TRECCOVID",
        "ArguAna",
        "TRECCOVID-NL",  # translation not trained on
        "ArguAna-NL",  # translation not trained on
    },
    adapted_from="BAAI/bge-base-en-v1.5",
    superseded_by=None,
    citation="""@software{balachandran2024medembed,
  author = {Balachandran, Abhinand},
  title = {MedEmbed: Medical-Focused Embedding Models},
  year = {2024},
  url = {https://github.com/abhinand5/MedEmbed}
}""",
)
avsolatorio__noinstruct_small_embedding_v0 = ModelMeta(
    name="avsolatorio/NoInstruct-small-Embedding-v0",
    model_type=["dense"],
    revision="b38747000553d8268915c95a55fc87e707c9aadd",
    release_date="2024-05-01",
    languages=["eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=33360000,
    n_embedding_parameters=11720448,
    memory_usage_mb=127,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/avsolatorio/NoInstruct-small-Embedding-v0",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
brahmairesearch__slx_v0_1 = ModelMeta(
    name="brahmairesearch/slx-v0.1",
    model_type=["dense"],
    revision="688c83fd1a7f34b25575a2bc26cfd87c11b4ce71",
    release_date="2024-08-13",
    languages=["eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=22713216,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=87,
    max_tokens=512.0,
    embed_dim=384,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/brahmairesearch/slx-v0.1",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
deepfile__embedder_100p = ModelMeta(
    name="deepfile/embedder-100p",
    model_type=["dense"],
    revision="aa02f08f11517977fbcdc94dc9dbf9a1ca152d9b",
    release_date="2023-07-24",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=None,
    n_embedding_parameters=192_001_536,
    memory_usage_mb=1061,
    max_tokens=514.0,
    embed_dim=768,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/deepfile/embedder-100p",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from="sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
    superseded_by=None,
)
infgrad__stella_base_en_v2 = ModelMeta(
    name="infgrad/stella-base-en-v2",
    model_type=["dense"],
    revision="c9e80ff9892d80b39dc54e30a7873f91ea161034",
    release_date="2023-10-19",
    languages=["eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=None,
    n_embedding_parameters=23_440_896,
    memory_usage_mb=None,
    max_tokens=512.0,
    embed_dim=None,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/infgrad/stella-base-en-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
malenia1__ternary_weight_embedding = ModelMeta(
    name="malenia1/ternary-weight-embedding",
    model_type=["dense"],
    revision="a1208fb7f646647bb62639fd2e1eb6cc2ef3738e",
    release_date="2024-10-23",
    languages=None,
    loader=sentence_transformers_loader,
    n_parameters=98688000,
    n_embedding_parameters=None,
    memory_usage_mb=158,
    max_tokens=512.0,
    embed_dim=1024,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "safetensors"],
    reference="https://huggingface.co/malenia1/ternary-weight-embedding",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from="ternary-weight-embedding",
    superseded_by=None,
)
omarelshehy__arabic_english_sts_matryoshka = ModelMeta(
    name="omarelshehy/arabic-english-sts-matryoshka",
    model_type=["dense"],
    revision="763d116fbe8bf7883c64635c862feeaa3768bb64",
    release_date="2024-10-13",
    languages=["ara-Arab", "eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=559890432,
    n_embedding_parameters=256_002_048,
    memory_usage_mb=2136,
    max_tokens=514.0,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/omarelshehy/arabic-english-sts-matryoshka",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from="FacebookAI/xlm-roberta-large",
    superseded_by=None,
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)
openbmb__minicpm_embedding = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_kwargs={
            # "attn_implementation": "flash_attention_2",
            "torch_dtype": torch.float16,
        },
        trust_remote_code=True,
        # https://huggingface.co/openbmb/MiniCPM-Embedding/blob/c0cb2de33fb366e17c30f9d53142ff11bc18e049/README.md?code=true#L405
    ),
    name="openbmb/MiniCPM-Embedding",
    model_type=["dense"],
    revision="c0cb2de33fb366e17c30f9d53142ff11bc18e049",
    release_date="2024-09-04",
    languages=["zho-Hans", "eng-Latn"],
    n_parameters=2724880896,
    n_embedding_parameters=282_822_912,
    memory_usage_mb=5197,
    max_tokens=512.0,
    embed_dim=2304,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "Transformers", "safetensors"],
    reference="https://huggingface.co/openbmb/MiniCPM-Embedding",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)

silma_ai__silma_embedding_matryoshka_v0_1 = ModelMeta(
    name="silma-ai/silma-embeddding-matryoshka-v0.1",
    model_type=["dense"],
    revision="a520977a9542ebdb8a7206df6b7ff6977f1886ea",
    release_date="2024-10-12",
    languages=["ara-Arab", "eng-Latn"],
    loader=sentence_transformers_loader,
    n_parameters=135193344,
    n_embedding_parameters=49_152_000,
    memory_usage_mb=516,
    max_tokens=512.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/silma-ai/silma-embeddding-matryoshka-v0.1",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from="/workspace/v3-matryoshka_aubmindlab-bert-base-arabertv02-2024-10-12_13-55-06/checkpoint-26250",
    superseded_by=None,
    citation="""@misc{silma2024embedding,
  author = {Abu Bakr Soliman, Karim Ouda, SILMA AI},
  title = {SILMA Embedding Matryoshka 0.1},
  year = {2024},
  publisher = {Hugging Face},
  howpublished = {https://huggingface.co/silma-ai/silma-embeddding-matryoshka-0.1},
}""",
)

sbert_chinese_general_v1 = ModelMeta(
    name="DMetaSoul/sbert-chinese-general-v1",
    model_type=["dense"],
    revision="bd27765956bcc2fcf682de0097819947ac10037e",
    release_date="2022-03-25",
    languages=["zho-Hans"],
    loader=sentence_transformers_loader,
    n_parameters=None,
    n_embedding_parameters=16_226_304,
    memory_usage_mb=None,  # Not visible on repo
    max_tokens=512,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "Transformers"],
    reference="https://huggingface.co/DMetaSoul/sbert-chinese-general-v1",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets={
        "PAWSX",
        "PawsXPairClassification",  # they do not specify which one
        # They might have trained on other datasets too, they don't say:
        # "trained on semantically similar datasets such as NLI, PAWS-X, PKU-Paraphrase-Bank, and STS."
    },
    superseded_by=None,
)

dmeta_embedding_zh_small = ModelMeta(
    name="DMetaSoul/Dmeta-embedding-zh-small",
    model_type=["dense"],
    revision="2050d3439a2f68999dd648c1697471acaac37a29",
    release_date="2024-03-25",
    languages=["zho-Hans"],
    loader=sentence_transformers_loader,
    n_parameters=int(74.2 * 1e6),
    n_embedding_parameters=16_226_304,
    memory_usage_mb=283,
    max_tokens=1024,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "Transformers", "safetensors"],
    reference="https://huggingface.co/DMetaSoul/Dmeta-embedding-zh-small/",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,  # They don't specify
    superseded_by=None,
)

xiaobu_embedding = ModelMeta(
    name="lier007/xiaobu-embedding",
    model_type=["dense"],
    revision="59c79d82eb5223cd9895f6eb8e825c7fa10e4e92",
    release_date="2024-01-09",
    languages=["zho-Hans"],
    loader=sentence_transformers_loader,
    n_parameters=int(326 * 1e6),
    n_embedding_parameters=21_635_072,
    memory_usage_mb=1244,
    max_tokens=512,
    embed_dim=1024,
    license="not specified",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "Transformers"],
    reference="https://huggingface.co/lier007/xiaobu-embedding",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,  # Finetuned from GTE, none of them disclose training data
    superseded_by=None,
    adapted_from="thenlper/gte-large-zh",
)

xiaobu_embedding_v2 = ModelMeta(
    name="lier007/xiaobu-embedding-v2",
    model_type=["dense"],
    revision="1912f2e59a5c2ef802a471d735a38702a5c9485e",
    release_date="2024-06-30",
    languages=["zho-Hans"],
    loader=sentence_transformers_loader,
    n_parameters=int(326 * 1e6),
    n_embedding_parameters=21_635_072,
    memory_usage_mb=1242,
    max_tokens=512,
    embed_dim=768,
    license="not specified",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/lier007/xiaobu-embedding-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,  # Finetuned from piccolo-embedding, none of them say
    superseded_by=None,
    adapted_from="sensenova/piccolo-base-zh",
)

yinka_embedding = ModelMeta(
    name="Classical/Yinka",
    model_type=["dense"],
    revision="59c79d82eb5223cd9895f6eb8e825c7fa10e4e92",
    release_date="2024-01-09",
    languages=["zho-Hans"],
    loader=sentence_transformers_loader,
    n_parameters=int(326 * 1e6),
    n_embedding_parameters=21_635_072,
    memory_usage_mb=1244,
    max_tokens=512,
    embed_dim=1024,
    license="not specified",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "Transformers"],
    reference="https://huggingface.co/Classical/Yinka",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,  # Not disclosed
    superseded_by=None,
    adapted_from="dunzhang/stella-mrl-large-zh-v3.5-1792d",
)
conan_embedding = ModelMeta(
    name="TencentBAC/Conan-embedding-v1",
    model_type=["dense"],
    revision="bb9749a57d4f02fd71722386f8d0f5a9398d7eeb",
    release_date="2024-08-22",
    languages=["zho-Hans"],
    loader=sentence_transformers_loader,
    n_parameters=int(326 * 1e6),
    n_embedding_parameters=21_635_072,
    memory_usage_mb=1242,
    max_tokens=512,
    embed_dim=768,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors"],
    reference="https://huggingface.co/Classical/Yinka",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    # source: https://arxiv.org/pdf/2408.15710
    training_datasets=None,  # They "scraped" things from the internet, we don't know, could be leakage
    superseded_by=None,
    citation="""@misc{li2024conanembeddinggeneraltextembedding,
  title={Conan-embedding: General Text Embedding with More and Better Negative Samples},
  author={Shiyu Li and Yang Tang and Shizhe Chen and Xi Chen},
  year={2024},
  eprint={2408.15710},
  archivePrefix={arXiv},
  primaryClass={cs.CL},
  url={https://arxiv.org/abs/2408.15710},
}""",
)

ember_v1 = ModelMeta(
    loader=sentence_transformers_loader,
    name="llmrails/ember-v1",
    model_type=["dense"],
    revision="5e5ce5904901f6ce1c353a95020f17f09e5d021d",
    release_date="2023-10-10",
    languages=["eng-Latn"],
    n_parameters=int(335 * 1e6),
    n_embedding_parameters=31_254_528,
    memory_usage_mb=1278,
    max_tokens=512,
    embed_dim=1024,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/llmrails/ember-v1",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    superseded_by=None,
    citation="""@misc{nur2024emberv1,
      title={ember-v1: SOTA embedding model},
      author={Enrike Nur and Anar Aliyev},
      year={2023},
}""",
)
