from .fa_mteb_pair_classification import (
    CExaPPC,
    FarsiParaphraseDetection,
    ParsinluEntail,
    ParsinluQueryParaphPC,
    SynPerChatbotRAGFAQPC,
    SynPerQAPC,
    SynPerTextKeywordsPC,
)
from .fars_tail import FarsTail

__all__ = [
    "CExaPPC",
    "FarsTail",
    "FarsiParaphraseDetection",
    "ParsinluEntail",
    "ParsinluQueryParaphPC",
    "SynPerChatbotRAGFAQPC",
    "SynPerQAPC",
    "SynPerTextKeywordsPC",
]
