from .no_rec_classification import NoRecClassification, NoRecClassificationV2
from .norwegian_parliament_classification import (
    NorwegianParliamentClassification,
    NorwegianParliamentClassificationV2,
)

__all__ = [
    "NoRecClassification",
    "NoRecClassificationV2",
    "NorwegianParliamentClassification",
    "NorwegianParliamentClassificationV2",
]
