from .indonesian_id_clickbait_classification import (
    IndonesianIdClickbaitClassification,
    IndonesianIdClickbaitClassificationV2,
)
from .indonesian_mongabay_conservation_classification import (
    IndonesianMongabayConservationClassification,
    IndonesianMongabayConservationClassificationV2,
)

__all__ = [
    "IndonesianIdClickbaitClassification",
    "IndonesianIdClickbaitClassificationV2",
    "IndonesianMongabayConservationClassification",
    "IndonesianMongabayConservationClassificationV2",
]
