from .spanish_passage_retrieval_s2p import SpanishPassageRetrievalS2P
from .spanish_passage_retrieval_s2s import SpanishPassageRetrievalS2S

__all__ = ["SpanishPassageRetrievalS2P", "SpanishPassageRetrievalS2S"]
